/* Configuration file compiler for extended goserve filter.         */
/* This REXX programm "compiles" the configuration file into        */
/* the os2forfl.80.                                                 */
/* For further information and all details see the documentation.   */
/* ================================================================ */
/* Written by Michael Warmuth for OS/2 Forum Austria.               */
/* (E-mail: Michael.Warmuth@wu-wien.ac.at).                         */
/* For using and copiing see the file copyright.doc.                */


/* Load the REXX Utility Functions */
if rxfuncquery('SysLoadFuncs') then do
   call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
   call SysLoadFuncs
end  /* Do */

/* Get argument */
parse arg filterfile .
if filterfile='' then filterfile = 'os2forfl.80'

/* Set global variables and constants */
parse source . . prog_dir
prog_dir = left(prog_dir,lastpos('\',prog_dir)) /* Directory of program */
filterfile = prog_dir||filterfile            /* Filter's filename */
conf_prg = prog_dir||'goconf.80'             /* Temporary filename */
backfile = left(filterfile,lastpos('.',filterfile))||'bak'  /* Backup filename */
reqq = 'OFAEGFLOGREQ'                        /* Extended logging queues */
cmdq = 'OFAEGFLOGCMD'

/* Warn user if filter can't be found */
if stream(filterfile,'c','query exists')='' then do
   say 'Looked for filter' filterfile', but couldn''t find it!'
   exit
end  /* Do */

/* Set global variables and constants (continued) */
dir = translate(SysIni('GOSERVE.INI','GoServe.80','DataDir'),'\','/')   /* GoServe's data path */
                           /* Data directory (root of all data directories) */
                           /* [must include drive and end in '/'] */
conf_dir = left(dir,lastpos('\',dir,length(dir)-2))'conf\'
                           /* Configuration directory = datadir\..\conf\ */
conf_sett = conf_dir'goconf.cnf'             /* Configuration file */

/* Look if extended logging queues exist already (and create them) */
newq = rxqueue('create',cmdq)                /* Create queue */
if newq\=cmdq then call rxqueue 'delete', newq  /* Delete queue if existed already */
newq = rxqueue('create',reqq)                /* Create queue */
if newq\=reqq then call rxqueue 'delete', newq  /* Delete queue if existed already */

/* Make backup of filter */
'@copy' filterfile backfile '1> nul 2> nul'

/* Read filter into memmory */
filter_cont = charin(filterfile,1,chars(filterfile))
parse var filter_cont first_part,
   '/* ----- Begin of automatic created program part ------------------------- */' .,
   '/* ----- End of automatic created program part --------------------------- */' last_part
   
call stream filterfile, 'c', 'close'
drop filter_cont

/* Delete temporary file */
if stream(conf_prg,'c','query exists')\='' then do
   call SysFileDelete conf_prg
end  /* Do */

/* Write filter content into temp file */
call charout conf_prg, first_part
drop first_part

/* Create header of compiled file */
call headcreate conf_prg

/* Compile configuration files */
call docompile conf_sett, conf_prg           /* Compile configuration file */

/* Create tail of compiled file */
call tailcreate conf_prg

/* Append end of original filter */
call charout conf_prg, last_part

/* Close compiled file */
call stream conf_prg, 'c', 'close'

/* Copy temp file to filter */
'@copy' conf_prg filterfile '1> nul 2> nul'
call SysFileDelete conf_prg

say "Compiling ended"

exit



/* Global procedures for compiling                                  */
/* ================================================================ */
/* docompile <name of text file>, <name of compiled file>           */
docompile: procedure

/* Get parameters */
parse arg conf_file, conf_prg .

/* Say info */
say 'Compiling file:' conf_file

/* Find first heading of a section */
found = 0

do while lines(conf_file)\=0                 /* With all lines of file */

   next_line = linein(conf_file)             /* Read next line */

   br_on = pos('[',next_line)                /* Find brackets and remark symbol */
   br_off = pos(']',next_line)
   rem_on = pos('#',next_line)

   if rem_on=0 then rem_on=999               /* If there is no remark in line */

   if br_on=1 & br_off\=0 & rem_on>br_off then do /* Section heading */
      parse var next_line '['section_name']' /* Get section name */
      say '   Found section:' section_name
      section_name = 'f.'strip(left(section_name,5))  /* Get first five chars */
      found = 1                              /* Set found flag */
   end  /* Do */

   else if found then do
      equ_pos = pos('=',next_line)           /* Find equal sign */

      if equ_pos\=0 & rem_on>equ_pos then do /* No remark line */
         parse var next_line key_name'='key_value'#'  /* Get key name and value */
         say '      Found key:' key_name
         key_name = strip(left(strip(key_name),5)) /* Format key name */
         key_value = strip(key_value)        /* Remove blanks from key value */
         key_value2 = ''

         /* Double single quotes */
         do while pos("'",key_value)
            parse var key_value tmp "'" key_value
            key_value2 = key_value2||tmp||"''"
         end /* do */
         key_value = key_value2||key_value
         drop tmp key_value2

         write_line = section_name'_'key_name "= '"key_value"'"   /* Build line */
         call lineout conf_prg, write_line   /* Wite line into compiled file */
      end  /* Do */

   end  /* Do */

end /* do */

/* Close configuration file */
call stream conf_file, 'c', 'close'

return 0



/* ================================================================ */
/* headcreate <name of compiled file>                               */
headcreate: procedure

/* Get name of of compiled file as parameter */
parse arg conf_prg .

/* Write out configuration server program's header */
call lineout conf_prg, "/* ----- Begin of automatic created program part ------------------------- */"
call lineout conf_prg, "/* Configuration server for extended goserve filter.                       */"
call lineout conf_prg, "/* Don't change this part of file per hand. Use the configuration file     */"
call lineout conf_prg, "/* and the compconf program instead.                                       */"
call lineout conf_prg, "/* For further information and all details see the documentation.          */"
call lineout conf_prg, "/* ======================================================================= */"
call lineout conf_prg, ""
call lineout conf_prg, "goconf: procedure expose f."
call lineout conf_prg, ""

return 0




/* ================================================================ */
/* tailcreate <name of compiled file>                               */
tailcreate: procedure

/* Get name of of compiled file as parameter */
parse arg conf_prg .

/* Write out configuration server program's tail */
call lineout conf_prg, ""
call lineout conf_prg, "return 0"
call charout conf_prg, "/* ----- End of automatic created program part --------------------------- */"

return 0
